@@ -44,6 +44,7 @@ typedef struct {
         GtkWidget *button;
         GtkWidget *image;
         GdkPixbuf *icon;
+        GtkWidget *about_dialog;
 
         PanelAppletOrient orient;
         int size;
@@ -255,6 +256,11 @@ applet_destroyed (GtkWidget       *applet,
 {
         g_object_unref (G_OBJECT (sdd->icon));
 
+	if (sdd->about_dialog) {
+		gtk_widget_destroy (sdd->about_dialog);
+		sdd->about_dialog =  NULL;
+	}
+
         g_free (sdd);
 }
 
@@ -282,8 +288,6 @@ show_desktop_applet_fill (PanelApplet *applet)
         sdd = g_new0 (ShowDesktopData, 1);
 
         sdd->applet = GTK_WIDGET (applet);
-        atk_obj = gtk_widget_get_accessible (sdd->button);
-        atk_object_set_name (atk_obj, _("Show Desktop Button"));
 
         file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
                                           "gnome-show-desktop.png", TRUE, NULL);
@@ -324,6 +328,8 @@ show_desktop_applet_fill (PanelApplet *applet)
         update_icon (sdd);
 
         sdd->button = gtk_toggle_button_new ();
+        atk_obj = gtk_widget_get_accessible (sdd->button);
+        atk_object_set_name (atk_obj, _("Show Desktop Button"));
 
         g_signal_connect (G_OBJECT (sdd->button), "button_press_event",
                           G_CALLBACK (do_not_eat_button_press), NULL);
@@ -394,7 +400,7 @@ display_help_dialog (BonoboUIComponent *uic,
         GError *error = NULL;
 
         egg_help_display_desktop_on_screen (
-                NULL, "show-desktop", "show-desktop", NULL,
+                NULL, "user-guide", "wgospanel.xml", "gospanel-564",
                 gtk_widget_get_screen (sdd->applet),
 		&error);
 
@@ -424,8 +430,6 @@ display_about_dialog (BonoboUIComponent *uic,
                       ShowDesktopData   *sdd,
                       const gchar       *verbname)
 {
-        static GtkWidget *about = NULL;
-
         static const gchar *authors[] = {
                 "Havoc Pennington <hp@redhat.com>",
                 NULL
@@ -437,14 +441,14 @@ display_about_dialog (BonoboUIComponent *uic,
         /* Translator credits */
         const char *translator_credits = _("translator_credits");
 
-        if (about) {
-                gtk_window_set_screen (GTK_WINDOW (about),
+        if (sdd->about_dialog) {
+                gtk_window_set_screen (GTK_WINDOW (sdd->about_dialog),
                                        gtk_widget_get_screen (sdd->applet));
-                gtk_window_present (GTK_WINDOW (about));
+                gtk_window_present (GTK_WINDOW (sdd->about_dialog));
                 return;
         }
 
-        about = gnome_about_new (_("Show Desktop Button"), VERSION,
+        sdd->about_dialog = gnome_about_new (_("Show Desktop Button"), VERSION,
                                  "Copyright \xc2\xa9 2002 Red Hat, Inc.",
                                  _("This button lets you hide all windows and show the desktop"),
                                  authors,
@@ -452,17 +456,17 @@ display_about_dialog (BonoboUIComponent *uic,
                                  strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
                                  sdd->icon);
 
-        gtk_window_set_wmclass (GTK_WINDOW (about), "show-desktop", "show-desktop");
-        gtk_window_set_screen (GTK_WINDOW (about),
+        gtk_window_set_wmclass (GTK_WINDOW (sdd->about_dialog), "show-desktop", "show-desktop");
+        gtk_window_set_screen (GTK_WINDOW (sdd->about_dialog),
                                gtk_widget_get_screen (sdd->applet));
 
         if (sdd->icon)
-                gtk_window_set_icon (GTK_WINDOW (about), sdd->icon);
+                gtk_window_set_icon (GTK_WINDOW (sdd->about_dialog), sdd->icon);
 
-        g_signal_connect (G_OBJECT(about), "destroy",
-                          (GCallback)gtk_widget_destroyed, &about);
+        g_signal_connect (G_OBJECT(sdd->about_dialog), "destroy",
+                          (GCallback)gtk_widget_destroyed, &sdd->about_dialog);
 
-        gtk_widget_show (about);
+        gtk_widget_show (sdd->about_dialog);
 }
 
 static void
