@@ -48,6 +48,8 @@ typedef struct {
 	GtkWidget    *frame;
 	GtkWidget    *image;
 	GtkWidget    *menu;
+	GtkWidget    *no_windows_item;
+	GtkWidget    *about_dialog;
 
 	GdkPixbuf    *icon_pixbuf;
 	WnckWindow   *icon_window;
@@ -67,7 +69,7 @@ window_menu_help (BonoboUIComponent *uic,
 	GError *error = NULL;
 
 	egg_help_display_desktop_on_screen (
-		NULL, "window-menu", "window-menu", NULL,
+		NULL, "user-guide", "wgosmetacity.xml", "gosmetacity-27",
 		gtk_widget_get_screen (window_menu->applet),
 		&error);
 
@@ -108,14 +110,13 @@ window_menu_about (BonoboUIComponent *uic,
 	const char *documenters [] = { NULL };
 	const char *translator_credits = _("translator_credits");
 
-	static GtkWidget *about = NULL;
 	GdkPixbuf        *pixbuf = NULL;
 	char             *file;
 
-	if (about) {
-		gtk_window_set_screen (GTK_WINDOW (about),
+	if (window_menu->about_dialog) {
+		gtk_window_set_screen (GTK_WINDOW (window_menu->about_dialog),
 				       gtk_widget_get_screen (window_menu->applet));
-		gtk_window_present (GTK_WINDOW (about));
+		gtk_window_present (GTK_WINDOW (window_menu->about_dialog));
 		return;
 	}
 
@@ -125,7 +126,7 @@ window_menu_about (BonoboUIComponent *uic,
 	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
 	g_free (file);
 
-	about = gnome_about_new (_("Window Menu"), VERSION,
+	window_menu->about_dialog = gnome_about_new (_("Window Menu"), VERSION,
 				 "Copyright \xc2\xa9 2003 Sun Microsystems, Inc.\n"
 				 "Copyright \xc2\xa9 2001 Free Software Foundation, Inc.\n"
 				 "Copyright \xc2\xa9 2000 Helix Code, Inc.",
@@ -135,18 +136,18 @@ window_menu_about (BonoboUIComponent *uic,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 
-	gtk_window_set_wmclass (GTK_WINDOW (about), "window-menu", "WindowMenu");
-	gtk_window_set_screen (GTK_WINDOW (about),
+	gtk_window_set_wmclass (GTK_WINDOW (window_menu->about_dialog), "window-menu", "WindowMenu");
+	gtk_window_set_screen (GTK_WINDOW (window_menu->about_dialog),
 			       gtk_widget_get_screen (window_menu->applet));
 
 	if (pixbuf) {
-		gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
+		gtk_window_set_icon (GTK_WINDOW (window_menu->about_dialog), pixbuf);
 		g_object_unref (pixbuf);
 	}
 
-	g_signal_connect (about, "destroy",
-			  (GCallback) gtk_widget_destroyed, &about);
-	gtk_widget_show (about);
+	g_signal_connect (window_menu->about_dialog, "destroy",
+			  (GCallback) gtk_widget_destroyed, &window_menu->about_dialog);
+	gtk_widget_show (window_menu->about_dialog);
 }
 
 static const BonoboUIVerb window_menu_verbs [] =
@@ -168,6 +169,15 @@ window_menu_destroy (GtkWidget  *widget,
 		g_object_unref (window_menu->icon_pixbuf);
 	window_menu->icon_pixbuf = NULL;
 
+	if (window_menu->no_windows_item)
+		g_object_unref (window_menu->no_windows_item);
+	window_menu->no_windows_item = NULL;
+
+	if (window_menu->about_dialog) {
+		gtk_widget_destroy (window_menu->about_dialog);
+		window_menu->about_dialog = NULL;
+	}
+
 	g_free (window_menu);
 }
 
@@ -382,6 +392,9 @@ window_menu_window_opened (WnckScreen *screen,
 			   WindowMenu *window_menu)
 {
 	if (window_menu->menu && GTK_WIDGET_VISIBLE (window_menu->menu)) {
+		if (window_menu->no_windows_item
+		    && GTK_WIDGET_VISIBLE (window_menu->no_windows_item))
+			gtk_widget_hide (window_menu->no_windows_item);
 		window_menu_add_window (window_menu, window);
 		gtk_menu_reposition (GTK_MENU (window_menu->menu));
 	}
@@ -509,6 +522,10 @@ window_menu_popup_menu (WindowMenu *window_menu,
 		gtk_container_remove (GTK_CONTAINER (window_menu->menu), l->data);
 	g_list_free (list);
 
+	if (window_menu->no_windows_item)
+		g_object_unref (window_menu->no_windows_item);
+	window_menu->no_windows_item = NULL;
+
 	g_signal_connect (window_menu->menu, "destroy",
 			  G_CALLBACK (window_menu_destroy_menu), window_menu);
 
@@ -533,13 +550,12 @@ window_menu_popup_menu (WindowMenu *window_menu,
 	}
 
 	if (!l) {
-		GtkWidget *item;
-		
-		item = gtk_menu_item_new_with_label (_("No Windows Open"));
+		window_menu->no_windows_item = gtk_menu_item_new_with_label (_("No Windows Open"));
 
-		gtk_widget_set_sensitive (item, FALSE);
-		gtk_widget_show (item);	
-		gtk_menu_shell_append (GTK_MENU_SHELL (window_menu->menu), item);
+		gtk_widget_set_sensitive (window_menu->no_windows_item, FALSE);
+		gtk_widget_show (window_menu->no_windows_item);	
+		gtk_menu_shell_append (GTK_MENU_SHELL (window_menu->menu),
+				       window_menu->no_windows_item);
 	}
 
 	gtk_widget_show (window_menu->menu);
@@ -615,10 +631,14 @@ window_menu_applet_fill (PanelApplet *applet)
 {
 	WindowMenu *window_menu;
 	GtkWidget  *alignment;
+	AtkObject  *atk_obj;
 
 	window_menu = g_new0 (WindowMenu, 1);
 
 	window_menu->applet = GTK_WIDGET (applet);
+	atk_obj = gtk_widget_get_accessible (window_menu->applet);
+	atk_object_set_name (atk_obj, _("Window Menu"));
+	atk_object_set_description (atk_obj, "Tool to switch between windows");
 
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 
