@@ -145,7 +145,11 @@ static void
 applet_remove_callback (GtkWidget  *widget,
 			AppletInfo *info)
 {
-	panel_profile_delete_object (info);
+
+	if (info->type == PANEL_OBJECT_DRAWER)
+		drawer_query_deletion (info->data);
+	else
+		panel_profile_delete_object (info);
 }
 
 static inline GdkScreen *
@@ -814,6 +818,7 @@ panel_applet_queue_applet_to_load (char            *id,
 
 	if (!toplevel_id) {
 		g_warning ("No toplevel on which to load object '%s'\n", id);
+		g_free (id);
 		return;
 	}
 
