@@ -211,24 +211,34 @@ nibble_on_file (const char *file)
 	FILE *fp;
 	mode_t old_mask;
 
+	if (file == NULL)
+		return NULL;
+
 	if (access (file, F_OK) == 0) {
+		int response;
+		char *utf8_name = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+
 		dialog = gtk_message_dialog_new
 			(GTK_WINDOW (toplevel),
 			 0 /* flags */,
 			 GTK_MESSAGE_QUESTION,
 			 GTK_BUTTONS_YES_NO,
 			 _("File %s already exists. Overwrite?"),
-			 file);
-		if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES) {
-			gtk_widget_destroy (dialog);
+			 utf8_name);
+		g_free (utf8_name);
+
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		if (response != GTK_RESPONSE_YES)
 			return NULL;
-		}
 	}
 
 	old_mask = umask(022);
 
 	fp = fopen (file, "w");
 	if (fp == NULL) {
+		char *utf8_name = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
 		dialog = gtk_message_dialog_new
 			(GTK_WINDOW (toplevel),
 			 0 /* flags */,
@@ -237,7 +247,8 @@ nibble_on_file (const char *file)
 			 _("Unable to create the file:\n"
 			   "\"%s\"\n"
 			   "Please check your permissions of "
-			   "the parent directory"), file ? file : "(null)");
+			   "the parent directory"), utf8_name);
+		g_free (utf8_name);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 
@@ -639,17 +650,17 @@ on_preview_expose_event (GtkWidget      *drawing_area,
 			 GdkEventExpose *event,
 			 gpointer        data)
 {
-	gdk_pixbuf_render_to_drawable (preview_image,
-				       drawing_area->window,
-				       drawing_area->style->white_gc,
-				       event->area.x,
-				       event->area.y,
-				       event->area.x,
-				       event->area.y,
-				       event->area.width,
-				       event->area.height,
-				       GDK_RGB_DITHER_NORMAL,
-				       0, 0);
+	gdk_draw_pixbuf (drawing_area->window,
+			 drawing_area->style->white_gc,
+			 preview_image,
+			 event->area.x,
+			 event->area.y,
+			 event->area.x,
+			 event->area.y,
+			 event->area.width,
+			 event->area.height,
+			 GDK_RGB_DITHER_NORMAL,
+			 0, 0);
 }
 
 void
@@ -735,6 +746,7 @@ gimme_file (const char *filename)
 		outfd = open (filename, O_CREAT|O_TRUNC|O_WRONLY, 0644);
 		if (outfd < 0) {
 			GtkWidget *dialog;
+			char *utf8_name = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
 			dialog = gtk_message_dialog_new
 				(GTK_WINDOW (toplevel),
 				 0 /* flags */,
@@ -743,7 +755,8 @@ gimme_file (const char *filename)
 				 _("Unable to create the file:\n"
 				   "\"%s\"\n"
 				   "Please check your permissions of "
-				   "the parent directory"), filename ? filename : "(null)");
+				   "the parent directory"), utf8_name);
+			g_free (utf8_name);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
 			close (infd);
@@ -753,6 +766,7 @@ gimme_file (const char *filename)
 		while ((bytes = read (infd, buf, sizeof (buf))) > 0) {
 			if (write (outfd, buf, bytes) != bytes) {
 				GtkWidget *dialog;
+				char *utf8_name = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
 				close (infd);
 				close (outfd);
 				unlink (filename);
@@ -762,7 +776,8 @@ gimme_file (const char *filename)
 					 GTK_MESSAGE_ERROR,
 					 GTK_BUTTONS_OK,
 					 _("Not enough room to write file %s"),
-					 filename ? filename : "(null)");
+					 utf8_name);
+				g_free (utf8_name);
 				gtk_dialog_run (GTK_DIALOG (dialog));
 				gtk_widget_destroy (dialog);
 				return FALSE;
@@ -789,16 +804,22 @@ on_ok_button_clicked (GtkWidget *widget,
 
 	button = glade_xml_get_widget (xml, "save_rbutton");
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
-		GtkWidget *entry;
-		GtkWidget *fileentry;
+		GtkWidget  *entry;
+		GtkWidget  *fileentry;
+		const char *tmp;
+
 		entry = glade_xml_get_widget (xml, "save_entry");
 		fileentry = glade_xml_get_widget (xml, "save_fileentry");
-		if (gimme_file (gtk_entry_get_text (GTK_ENTRY (entry)))) {
+ 		tmp = gtk_entry_get_text (GTK_ENTRY (entry));
+ 		file = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
+ 		if (gimme_file (file)) {
 			gnome_entry_prepend_history (GNOME_ENTRY (gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (fileentry))),
 						     TRUE, gtk_entry_get_text (GTK_ENTRY (entry)));
 			gtk_main_quit ();
 		}
+		g_free (file);
 		setup_busy (FALSE);
+
 		return;
 	}
 
@@ -858,7 +879,7 @@ on_help_button_clicked (GtkWidget *widget, gpointer data)
                 dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
                         GTK_DIALOG_DESTROY_WITH_PARENT,
                         GTK_MESSAGE_ERROR,
-                        GTK_BUTTONS_CLOSE,
+                        GTK_BUTTONS_OK,
                         _("There was an error displaying help: \n%s"),
                         error->message);
 
@@ -918,7 +939,7 @@ find_toplevel_window (int depth, Window xid, gboolean *keep_going)
 	if (XGetWindowProperty (GDK_DISPLAY (), xid, wm_state, 0, 1,
 				False, AnyPropertyType, &actual_type,
 				&actual_format, &nitems, &bytes_after,
-				(guchar **) &prop) == Success
+				(gpointer) &prop) == Success
 	    && prop != NULL && actual_format == 32 && prop[0] == NormalState)
 	{
 		/* Found a top-level window */
@@ -961,13 +982,15 @@ find_toplevel_window (int depth, Window xid, gboolean *keep_going)
 static gboolean
 take_window_shot (void)
 {
-	GdkWindow *window;
+	GdkWindow *window, *toplevel_window;
 	Display *disp;
 	Window w, root, child, toplevel;
 	int unused;
 	guint mask;
+	gint x_real_orig, y_real_orig;
 	gint x_orig, y_orig;
 	gint x = 0, y = 0;
+	gint real_width, real_height;
 	gint width, height;
 	XClassHint class_hint;
 	gchar *result = NULL;
@@ -1009,11 +1032,19 @@ take_window_shot (void)
 
 			XFree (class_hint.res_name);
 		}
+
+		/* Force window to be shown */
+		toplevel_window	 = gdk_window_foreign_new (toplevel);
+		gdk_window_show (toplevel_window);
 	}
 
-	gdk_drawable_get_size (window, &width, &height);
-	gdk_window_get_origin (window, &x_orig, &y_orig);
+	gdk_drawable_get_size (window, &real_width, &real_height);
+	gdk_window_get_origin (window, &x_real_orig, &y_real_orig);
 
+	x_orig = x_real_orig;
+	y_orig = y_real_orig;
+	width = real_width;
+	height = real_height;
 	
 	if (x_orig < 0) {
 		x = - x_orig;
@@ -1044,21 +1075,48 @@ take_window_shot (void)
 		gdk_pixbuf_fill (screenshot, 0);
 	
 		for (i = 0; i < rectangle_count; i++) {
-			for (y = rectangles[i].y; y < rectangles[i].y + rectangles[i].height; y++) {
+			gint rec_x, rec_y;
+			gint rec_width, rec_height;
+
+			rec_x = rectangles[i].x;
+			rec_y = rectangles[i].y;
+			rec_width = rectangles[i].width;
+			rec_height = rectangles[i].height;
+
+			if (x_real_orig < 0) {
+				rec_x += x_real_orig;
+				rec_x = MAX(rec_x, 0);
+				rec_width += x_real_orig;
+			}
+			if (y_real_orig < 0) {
+				rec_y += y_real_orig;
+				rec_y = MAX(rec_y, 0);
+				rec_height += y_real_orig;
+			}
+
+			if (x_orig + rec_x + rec_width > gdk_screen_width ())
+				rec_width = gdk_screen_width () - x_orig - rec_x;
+			if (y_orig + rec_y + rec_height > gdk_screen_height ())
+				rec_height = gdk_screen_height () - y_orig - rec_y;
+
+			for (y = rec_y; y < rec_y + rec_height; y++) {
 				guchar *src_pixels, *dest_pixels;
+				gboolean has_alpha = gdk_pixbuf_get_has_alpha (tmp);
 				
 				src_pixels = gdk_pixbuf_get_pixels (tmp) +
 					y * gdk_pixbuf_get_rowstride(tmp) +
-					rectangles[i].x * (gdk_pixbuf_get_has_alpha (tmp) ? 4 : 3);
+					rec_x * (has_alpha ? 4 : 3);
 				dest_pixels = gdk_pixbuf_get_pixels (screenshot) +
 					y * gdk_pixbuf_get_rowstride (screenshot) +
-					rectangles[i].x * 4;
+					rec_x * 4;
 				
-				for (x = rectangles[i].x; x < rectangles[i].x + rectangles[i].width; x++) {
+				for (x = 0; x < rec_width; x++) {
 					*dest_pixels++ = *src_pixels ++;
 					*dest_pixels++ = *src_pixels ++;
 					*dest_pixels++ = *src_pixels ++;
 					*dest_pixels++ = 255;
+					if (has_alpha)
+						src_pixels++;
 				}
 			}
 		}
@@ -1208,6 +1266,7 @@ main (int argc, char *argv[])
 	gint width, height; 
 	guint delay = 0;
 	gboolean normal_web_dir = TRUE;
+	gchar *utf8_name;
 	
 	struct poptOption opts[] = {
 		{"window", '\0', POPT_ARG_NONE, &window, 0, N_("Grab a window instead of the entire screen"), NULL},
@@ -1336,12 +1395,14 @@ main (int argc, char *argv[])
 		desktop_dir = g_strdup (home_dir);
 	else
 		desktop_dir = g_strconcat (home_dir, G_DIR_SEPARATOR_S,
-					   ".gnome-desktop", NULL);
+					   "Desktop", NULL);
 	g_object_unref (gconf_client);
 	
 	file = add_file_to_path (home_dir);
-	gtk_entry_set_text (GTK_ENTRY (save_entry), file);
+	utf8_name = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+	gtk_entry_set_text (GTK_ENTRY (save_entry), utf8_name);
 	g_free (file);
+	g_free (utf8_name);
 
 	if (!stat (web_dir, &s) && S_ISDIR (s.st_mode)) {
 		GtkWidget *cbutton;
