@@ -57,6 +57,7 @@ struct _PanelAppletPrivate {
 	BonoboControl              *control;
 	BonoboPropertyBag          *prop_sack;
 	BonoboItemHandler          *item_handler;
+	GConfClient                *client;
 
 	char                       *iid;
 	GClosure                   *closure;
@@ -182,9 +183,8 @@ panel_applet_add_preferences (PanelApplet  *applet,
 			      const gchar  *schema_dir,
 			      GError      **opt_error)
 {
-	GConfClient  *client;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
+	GError **error = NULL;
+	GError  *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 	g_return_if_fail (schema_dir != NULL);
@@ -197,22 +197,17 @@ panel_applet_add_preferences (PanelApplet  *applet,
 	else
 		error = &our_error;
 
-	client = gconf_client_get_default ();
-
 	panel_applet_associate_schemas_in_dir (
-		client, applet->priv->prefs_key, schema_dir, error);
+		applet->priv->client, applet->priv->prefs_key, schema_dir, error);
 
 	if (!opt_error && our_error) {
 		g_warning (G_STRLOC ": failed to add preferences from '%s' : '%s'",
 			   schema_dir, our_error->message);
 		g_error_free (our_error);
 	}
-
-	gconf_client_add_dir (
-		client, applet->priv->prefs_key, GCONF_CLIENT_PRELOAD_NONE, NULL);
 }
 
-gchar *
+char *
 panel_applet_get_preferences_key (PanelApplet *applet)
 {
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
@@ -223,6 +218,31 @@ panel_applet_get_preferences_key (PanelApplet *applet)
 	return g_strdup (applet->priv->prefs_key);
 }
 
+static void
+panel_applet_set_preferences_key (PanelApplet *applet,
+				  const char  *prefs_key)
+{
+	g_return_if_fail (PANEL_IS_APPLET (applet));
+
+	if (applet->priv->prefs_key) {
+		gconf_client_remove_dir (applet->priv->client,
+					 applet->priv->prefs_key,
+					 NULL);
+
+		g_free (applet->priv->prefs_key);
+		applet->priv->prefs_key = NULL;
+	}
+
+	if (prefs_key) {
+		applet->priv->prefs_key = g_strdup (prefs_key);
+
+		gconf_client_add_dir (applet->priv->client,
+				      applet->priv->prefs_key,
+				      GCONF_CLIENT_PRELOAD_RECURSIVE,
+				      NULL);
+	}
+}
+
 PanelAppletFlags
 panel_applet_get_flags (PanelApplet *applet)
 {
@@ -237,7 +257,10 @@ panel_applet_set_flags (PanelApplet      *applet,
 {
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	bonobo_pbclient_set_short (BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_FLAGS, flags, NULL);
+	if (applet->priv->prop_sack != NULL)
+		bonobo_pbclient_set_short (BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_FLAGS, flags, NULL);
+	else
+		applet->priv->flags = flags;
 }
 
 void
@@ -247,9 +270,12 @@ panel_applet_set_size_hints (PanelApplet      *applet,
 			     int               base_size)
 {
 	CORBA_sequence_CORBA_long *seq;
+	CORBA_Environment          env;
 	CORBA_any                  any;
 	int                        i;
 
+	CORBA_exception_init (&env);
+
 	seq = CORBA_sequence_CORBA_long__alloc ();
 	seq->_length = seq->_maximum = n_elements;
 	seq->_release = CORBA_TRUE;
@@ -265,9 +291,11 @@ panel_applet_set_size_hints (PanelApplet      *applet,
 	Bonobo_PropertyBag_setValue (BONOBO_OBJREF (applet->priv->prop_sack),
 				     PROPERTY_SIZE_HINTS,
 				     &any,
-				     NULL);
+				     &env);
 
 	CORBA_free (seq);
+
+	CORBA_exception_free (&env);
 }
 
 guint
@@ -357,15 +385,26 @@ panel_applet_finalize (GObject *object)
 {
 	PanelApplet *applet = PANEL_APPLET (object);
 
+	panel_applet_set_preferences_key (applet, NULL);
+
+	if (applet->priv->client)
+		g_object_unref (applet->priv->client);
+	applet->priv->client = NULL;
+
 	if (applet->priv->prop_sack)
 		bonobo_object_unref (
 			BONOBO_OBJECT (applet->priv->prop_sack));
+	applet->priv->prop_sack = NULL;
 
 	g_free (applet->priv->size_hints);
 	g_free (applet->priv->prefs_key);
 	g_free (applet->priv->background);
 	g_free (applet->priv->iid);
 
+	if (applet->priv->closure)
+		g_closure_unref (applet->priv->closure);
+	applet->priv->closure = NULL;
+
 	g_free (applet->priv);
 	applet->priv = NULL;
 
@@ -1129,7 +1168,7 @@ panel_applet_item_handler_get_object (BonoboItemHandler *handler,
 			continue;
 
 		if (!strcmp (option->key, "prefs_key") && !applet->priv->prefs_key)
-			applet->priv->prefs_key = g_strdup (option->value);
+			panel_applet_set_preferences_key (applet, option->value);
 
 		else if (!strcmp (option->key, "background"))
 			bonobo_pbclient_set_string (BONOBO_OBJREF (applet->priv->prop_sack),
@@ -1311,11 +1350,12 @@ panel_applet_instance_init (PanelApplet      *applet,
 {
 	applet->priv = g_new0 (PanelAppletPrivate, 1);
 
-	applet->priv->bound        = FALSE;
+	applet->priv->client = gconf_client_get_default ();
 
-	applet->priv->flags        = PANEL_APPLET_FLAGS_NONE;
-	applet->priv->orient       = PANEL_APPLET_ORIENT_UP;
-	applet->priv->size         = GNOME_Vertigo_PANEL_MEDIUM;
+	applet->priv->bound  = FALSE;
+	applet->priv->flags  = PANEL_APPLET_FLAGS_NONE;
+	applet->priv->orient = PANEL_APPLET_ORIENT_UP;
+	applet->priv->size   = GNOME_Vertigo_PANEL_MEDIUM;
 
 	applet->priv->moving_focus_out = FALSE;
 
